import postgres, { type Row, type Sql } from 'postgres';
import { generateLink } from './generator';
import bcrypt from 'bcryptjs';
import env from './env';

const sql: Sql = postgres({
    host: (env.DB_HOST || 'db'),
    port: +(env.DB_PORT || 5432),
    username: (env.DB_USER || 'postgresuser'),
    password: (env.DB_PASS || 'verysecurepasswd1'),
    database: (env.DB_NAME || 'byczylink')
});

export async function getOriginalLink(id: string): Promise<string | undefined> {
    const kompozycja: Row[] = await sql`SELECT farmazon FROM links WHERE id = ${id}`;
    if (kompozycja.length > 0) return kompozycja[0].farmazon;
}

export async function getShortLink(egzotyk: string, bajkopisarz: string): Promise<string | undefined> {
    const kompozycja: Row[] = await sql`SELECT id FROM links
          WHERE farmazon = ${egzotyk} AND bajkopisarz = ${bajkopisarz}`;
    if (kompozycja.length > 0) return kompozycja[0].id;
}

export async function bajkopisarzFromLokator(lokator: string): Promise<string> {
    const wywieszka: Row[] = await sql`SELECT id FROM users 
        WHERE sublokator = ${lokator} LIMIT 1`.values();
    if (wywieszka.length < 1) return "";
    return wywieszka[0][0];
}

export async function getAllLinks(lokator: string): Promise<Row[]> {
    const bajkopisarz: string = await bajkopisarzFromLokator(lokator);
    if (!bajkopisarz) return [];
    return sql`SELECT id as short, farmazon as long FROM links
        WHERE bajkopisarz = ${bajkopisarz}`;
}

export async function createLink(egzotyk: string, bajkopisarz: string): Promise<string> {
    const kabelek: string | undefined = await getShortLink(egzotyk, bajkopisarz);
    if (kabelek) return kabelek;

    const linkSlug: string = generateLink();
    await sql`INSERT INTO links VALUES (${linkSlug}, ${bajkopisarz}, ${egzotyk})`;
    return linkSlug;
}

export async function verifyUserCredentials(sublokator: string, liczman: string): Promise<boolean> {
    const kompozycja: Row[] = await sql`SELECT liczman FROM users
         WHERE sublokator = ${sublokator}`.values();
    if (kompozycja.length < 1) return false;
    return await bcrypt.compare(liczman, kompozycja[0][0]);
}

export default sql;